# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

######################################################
############# IMPORTAZIONE MODULI ####################
######################################################
from gi.repository import Gtk
from preventares_lib.helpers import get_builder
################################################################################
############ SUPPORTO INTERNAZIONALIZZAZIONE  ##################################
################################################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
################################################################################
############# DIALOG ###########################################################
################################################################################
class ManodoperaDialog(Gtk.Dialog):
    __gtype_name__ = "ManodoperaDialog"

    def __new__(cls, preventivo):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ManodoperaDialog object.
        """
        builder = get_builder('ManodoperaDialog')
        new_object = builder.get_object('manodopera_dialog')
        new_object.finish_initializing(builder, preventivo)
        return new_object

    def __init__(self, preventivo=None):
        """""" 
        pass

    def finish_initializing(self, builder, preventivo):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a ManodoperaDialog object with it in order to
        finish initializing the start of the new ManodoperaDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.builder.connect_signals(self)
        self.preventivo = preventivo
        # ottieni un riferimento agli elementi dell'interfaccia grafica
        self.pulsante_aggiungi = builder.get_object("button3")
        self.pulsante_aggiungi.connect('clicked', self.aggiungi_rigo_mo)
        self.pulsante_rimuovi = builder.get_object("button4")
        self.pulsante_rimuovi.connect('clicked', self.elimina_rigo_mo)
        self.label_calcola = builder.get_object("label2")
        ###########################################
        # treeview MANODOPERA
        self.liststore_mo = Gtk.ListStore(int, str, float, float, str)
        self.treeview_mo = builder.get_object("treeview1")
        self.treeview_mo.set_model(self.liststore_mo)
        # imposta la possibilita di selezione multipla
        selection = self.treeview_mo.get_selection()
        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        # create the TreeViewColumns to display the data
        self.treeview_mo.columns = [None]*5
        self.treeview_mo.columns[0] = Gtk.TreeViewColumn('N.')
        self.treeview_mo.columns[1] = Gtk.TreeViewColumn('Tipo di Manodopera')
        self.treeview_mo.columns[2] = Gtk.TreeViewColumn('Costo Orario')
        self.treeview_mo.columns[3] = Gtk.TreeViewColumn('Perc.(%)')
        self.treeview_mo.columns[4] = Gtk.TreeViewColumn('Note')
        for n in range(5):
            # add columns to treeview
            self.treeview_mo.append_column(self.treeview_mo.columns[n])
            # create a CellRenderers to render the data
            self.treeview_mo.columns[n].cell = Gtk.CellRendererText()
            # set the cells as editable
            self.treeview_mo.columns[n].cell.set_property('editable', True)
            self.treeview_mo.columns[n].cell.connect('edited', self.cell_mo_edited, n)
            # add the cells to the columns
            self.treeview_mo.columns[n].pack_start(self.treeview_mo.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_mo.columns[n].add_attribute(self.treeview_mo.columns[n].cell, 'text', n)
            # set column resizable
            self.treeview_mo.columns[n].set_resizable(True)
        # aggiorna il liststore della manodopera
        self.Aggiorna_finestra_dialogo()

    def Aggiorna_finestra_dialogo(self):
        """aggiorna la treeview e l'interfaccia grafica del dialog"""
        self.liststore_mo.clear()
        # aggiorna il liststore della manodopera
        lista = self.preventivo.get_table_manodopera()
        for mo in lista:
            self.liststore_mo.append(mo)
        # calcola la media, non aggiorna i dati generali
        mo = self.preventivo.calcola_media_manodopera (update_dg = False)
        if mo is None:
            mo = self.preventivo.dati_generali_list()[1]
        # aggiorna l'etichetta
        self.label_calcola.set_text(str(mo))

    def aggiungi_rigo_mo(self, *arg):
        """aggiunge un tipo di manodopera"""
        self.preventivo.insert_manodopera ("Manodopera operaio specializzato", 
                        costo=None, perc=None, note=None)
        # aggiorna l'interfaccia grafica
        self.Aggiorna_finestra_dialogo()

    def elimina_rigo_mo(self, *arg):
        """cancella un tipo di manodopera"""
        selection = self.treeview_mo.get_selection()
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            iter = model.get_iter(paths[0])
            key = int(model.get_value(iter, 0))
            self.preventivo.delete_manodopera(key)
        # aggiorna l'interfaccia grafica
        self.Aggiorna_finestra_dialogo()

    def cell_mo_edited(self, cell, path, new_text, col_nr):
        """Called when a text cell is edited. It puts the new text in the model so that it is displayed properly."""
        model = self.treeview_mo.get_model()
        model[path][col_nr] = new_text
        iter = model.get_iter(path)
        key = int(model.get_value(iter, 0))
        nome = model.get_value(iter, 1)
        costo = model.get_value(iter, 2)
        perc = model.get_value(iter, 3)
        note = model.get_value(iter, 4)
        if col_nr in [2, 3]:
            new_text = self.__verifica_testo_immesso(new_text)
        if col_nr == 1:
            nome = new_text
        elif col_nr == 2:
            costo = new_text
        elif col_nr == 3:
            perc = new_text
        elif col_nr == 4:
            note = new_text
        # aggiorna la tabella della manodopera
        result = self.preventivo.update_manodopera (key, nome=nome, costo=costo,
                                                    perc=perc, note=note)
        # aggiorna l'interfaccia grafica
        self.Aggiorna_finestra_dialogo()
        return True

    def __verifica_testo_immesso (self, text):
        """Verifica se il dato immesso è valido o avvisa l'utente tramite un messaggio di errore
           questo metodo è valido solamente per stringhe di testo"""
        text = text.replace(',', '.')
        try:
            text = float(text)
        except ValueError:
            if not self.message(self, "errore", "Valore non corretto. Immettere una numero decimale o intero"):
                elenco_caratteri_ok = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '.']
                for carattere in list(text):
                    if carattere not in elenco_caratteri_ok:
                        text = text.replace(carattere, '')
                try:
                    text = float(text)
                except ValueError:
                    text = 0.0
        return text

    def ok(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.RESONSE_OK from run().
        """
        # calcola la media, non aggiorna i dati generali
        mo = self.preventivo.calcola_media_manodopera (update_dg = False)
        if self.message(self, "avviso", "Attenzione si stà per modificare il valore della manodopera del computo a %s" % str(mo)):
            # calcola la media e aggiorna i dati generali
            self.preventivo.calcola_media_manodopera()

    def cancel(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns Gtk.RESPONSE_CANCEL for run()
        """
        pass

################################################################################
###  FINESTRA MESSAGGI UTENTE ##################################################
################################################################################
    def message(self, widget, tipo, testo, data=None):
        """display a message windows"""
        buttons = Gtk.ButtonsType.OK_CANCEL
        if tipo == "info":
           tipo = Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        elif tipo == "avviso":
           tipo = Gtk.MessageType.WARNING
        elif tipo == "domanda":
           tipo = Gtk.MessageType.QUESTION
        elif tipo == "errore":
           tipo = Gtk.MessageType.ERROR
           buttons = Gtk.ButtonsType.CLOSE
        else:
           tipo == Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        message = Gtk.MessageDialog(parent=None, flags=0, type=tipo, buttons = buttons, message_format=testo)
        message.format_secondary_text(_("Premere un pulsante per continuare"))
        response = message.run()
        if response == Gtk.ResponseType.OK:
           response = True
        else:
           response = False 
        message.destroy()
        return response
################################################################################
###  NON MODIFICARE ############################################################
################################################################################
if __name__ == "__main__":
    dialog = ManodoperaDialog()
    dialog.show()
    Gtk.main()
