# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

######################################################
########### MODULI E LIBRERIE BASE ###################
######################################################
import sys
import os
import urllib
import ConfigParser as configparser #python 3.2 (modulo rinominato configparser)
from gi.repository import Gtk, Gdk, GdkPixbuf, GObject # pylint: disable=E0611
from preventares.AboutPreventaresDialog import AboutPreventaresDialog
from preventares.PreferencesPreventaresDialog import PreferencesPreventaresDialog
######################################################
############ IMPOSTAZIONI LOGGINGS ###################
######################################################
import logging
logger = logging.getLogger('preventares')
######################################################
########### MODULI PER INTERNAZIONALIZZAZIONE ########
######################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
######################################################
########### IMPORTAZIONE INTERFACCIA GRAFICA #########
######################################################
from preventares import (AboutPreventaresDialog, MetelWindow, ComputoWindow,
                         gtkcrashhandler, OptionChooserDialog, PrintoptionDialog)
from preventares_lib.helpers import get_builder, show_uri, get_help_uri
from preventares.FileChooserDialog import FileChooserDialog

######################################################
########### IMPORTAZIONE IMPOSTAZIONI DI DEFAULT #####
######################################################
from preventares_lib.preventaresconfig import (get_data_file, 
                                               get_config_path, 
                                               get_config_file, 
                                               get_archive_path,
                                               get_print_settings_file)
# importazione delle impostazioni di default della libreria Preventa_lib
from preventares_lib.preventa_lib import DEFAULT_SETTINGS as PREVENTA_LIB_DEFAULT_SETTINGS
# importazione delle impostazioni di default dell'interfaccia grafica (globali x tutto il programma)
from preventares.ComputoWindow import DEFAULT_SETTINGS as UI_DEFAULT_SETTINGS
from preventares.ComputoWindow import SETTINGS_TRANSLATION as UI_SETTINGS_TRANSLATION

######################################################
###### INIZIALIZZAZIONE VARIBILI E IMPOSTAZIONI ######
######################################################
# nome dell'applicazione
APP_NAME='preventares'
# Versione dell'applicazione
VERSION='11.06.1'
# Sezioni del file di configurazione
SECTION_UI = "PreventARES"
SECTION_LIB = "PreventaLib"

######################################################
########### FINESTRA PRINCIPALE DI SCELTA ############
######################################################
# See preventares_lib.Window.py for more details about how this class works
class PreventaresWindow(Gtk.Window):
    __gtype_name__ = "PreventaresWindow"
   
    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your inialization code in finish_intializing
   
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ComputoWindow object.
        """
        builder = get_builder('PreventaresWindow')
        new_object = builder.get_object("preventares_window")
        new_object.finish_initializing(builder)
        return new_object

    def __init__(self):
        pass
        
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        # set up the signals
        builder.connect_signals(self)
        
        self.AboutDialog = AboutPreventaresDialog
        self.PreferencesDialog = PreferencesPreventaresDialog
        #########################################################
        # Code for other initialization actions should be added here.
        #########################################################
        self.FileComputo = None
        self.FileArchivio = None
        # avvia il gestore di crash
        gtkcrashhandler.initialize(APP_NAME)
        # legge i file di configurazione (o li crea se non esistono)
        
        self.computo_settings = self.get_main_settings(UI_DEFAULT_SETTINGS,
                                                       SECTION_UI)
        self.preventa_settings = self.get_main_settings(
                                 PREVENTA_LIB_DEFAULT_SETTINGS, SECTION_LIB)
        # verifica se esiste il file l'archivio e lo crea se non esiste
        if not os.path.isfile(self.computo_settings['main_archive_filename']):
            from preventares_lib.preventa_lib import Preventivo
            new_archivio = Preventivo(self.computo_settings['main_archive_filename'])
            new_archivio.save_database()
            new_archivio.connection_shutdown()
        else:
            pass #logger.debug("Archive {0} existant".format(self.computo_settings['main_archive_filename']))
        # verifica se esiste il file metel e lo crea se non esiste
        if not os.path.isfile(self.computo_settings['metel_archive_filename']):
            from preventares_lib.metel_lib import MetelClass
            new_metel = MetelClass(self.computo_settings['metel_archive_filename'])
            new_metel.save_database()
            new_metel.connection_shutdown()
        else:
            pass #logger.debug("Metel archive {0} existant".format(self.computo_settings['metel_archive_filename']))
        # registra le nuove icone stock
        icon_info = [
                    # icone base
                    ('preventa-dati-generali', get_data_file('media', 'DatiGenerali.svg')),
                    ('preventa-computo-metrico', get_data_file('media', 'ComputoMetrico.svg')),
                    ('preventa-elenco-prezzi', get_data_file('media', 'ElencoPrezzi.svg')),
                    ('preventa-listino', get_data_file('media', 'ListinoMetel.svg')),
                    ('preventa-archive', get_data_file('media', 'ArchivioPrincipale.svg')),
                    # icone computo
                    ('preventa-new-computo', get_data_file('media', 'DocumentoNewMt.svg')),
                    ('preventa-open-computo', get_data_file('media', 'DocumentoEdit.svg')),
                    # articoli
                    ('preventa-new-articolo', get_data_file('media', 'DocumentoNew.svg')),
                    ('preventa-insert-articolo', get_data_file('media', 'DocumentoIn.svg')),
                    ('preventa-erase-articolo', get_data_file('media', 'DocumentoErase.svg')),
                    # icone archivio
                    ('preventa-open-archive', get_data_file('media', 'PrezziarioEdit.svg')),
                    ('preventa-edit-archive', get_data_file('media', 'PrezziarioEdit.svg')),
                    ('preventa-insert-archive', get_data_file('media', 'PrezziarioIn.svg')),
                    ('preventa-export-archive', get_data_file('media', 'PrezziarioOut.svg')),
                    ('preventa-erase-archive', get_data_file('media', 'PrezziarioErase.svg')),
                    # altre funzioni
                    ('preventa-reload-computo', get_data_file('media', 'PrezziarioReload.svg')),
                    # icone impostazioni
                    ('preventa-set-program', get_data_file('media', 'configura.svg')),
                    ('preventa-set-database', get_data_file('media', 'db-config.svg'))]
        self.register_iconsets (icon_info)
        # imposta le icone nei pulsanti
        img_new_cm = builder.get_object("image2")
        img_new_cm.set_from_stock('preventa-new-computo', Gtk.IconSize.DIALOG)
        img_open_cm = builder.get_object("image3")
        img_open_cm.set_from_stock('preventa-open-computo', Gtk.IconSize.DIALOG)
        img_archive = builder.get_object("image4")
        img_archive.set_from_stock('preventa-archive', Gtk.IconSize.DIALOG)
        img_listino = builder.get_object("image5")
        img_listino.set_from_stock('preventa-listino', Gtk.IconSize.DIALOG)
        img_set_cm = builder.get_object("image6")
        img_set_cm.set_from_stock('preventa-set-program', Gtk.IconSize.DIALOG)
        img_set_lib = builder.get_object("image7")
        img_set_lib.set_from_stock('preventa-set-database', Gtk.IconSize.DIALOG)
        ####################################################################
        # Supporto Drag'n Drop per apertura file computo
        ####################################################################
        # Costanti per il supporto Drag n'Drop per apertura file computo
        self.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
        self.connect("drag_data_received", self.window_drag_data_received)
        self.drag_dest_add_uri_targets()        
        
        
################################################################################
###  REGISTRA NUOVE ICONE PER L'APPLICAZIONE ###################################
################################################################################
    def register_iconsets(self, icon_info):
        """"""
        iconfactory = Gtk.IconFactory()
        stock_ids = Gtk.stock_list_ids()
        for stock_id, file in icon_info:
            # only load image files when our stock_id is not present
            if stock_id not in stock_ids:
                pixbuf = GdkPixbuf.Pixbuf.new_from_file(file)
                iconset = Gtk.IconSet(pixbuf)
                iconfactory.add(stock_id, iconset)
        iconfactory.add_default()


################################################################################
###  APERTURA FILE COMPUTO CON DRAG'N DROP #####################################
################################################################################
    def window_drag_data_received(self, widget, context, x, y, selection, info, time):
        #get the path to file
        def get_file_path_from_dnd_dropped_uri(uri):
            path = ""
            if uri.startswith('file:\\\\\\'): # windows
                path = uri[8:] # 8 is len('file:///')
            elif uri.startswith('file://'): # nautilus, rox
                path = uri[7:] # 7 is len('file://')
            elif uri.startswith('file:'): # xffm
                path = uri[5:] # 5 is len('file:')
            path = urllib.url2pathname(path) # escape special chars
            path = path.strip('\r\n\x00') # remove \r\n and NULL
            return path
        # ... 
        uris = selection.get_uris()
        i = len(uris)
        for uri in uris:
            path = get_file_path_from_dnd_dropped_uri(uri)
            if os.path.isfile(path): # is it file?
                try:
                    computo = ComputoWindow.ComputoWindow(filename=path, 
                              filearchivio = self.get_file_archivio(), 
                              filelistino = self.get_file_listino(),
                              preventa_settings= self.preventa_settings, 
                              computo_settings=self.computo_settings)
                    self.hide()
                    computo.show()
                except:
                    self.message(self, "errore", _("Il file {0} non è un archivio PreventARES valido".format(path)))
                    i -= 1
        # aumenta i livelli di Gtk.main()
        while i > 0:
            i -= 1
            Gtk.main()
        self.show()
        return True

    

#################################################################
########### CREAZIONE/LETTURA FILE DI CONFIGURAZIONE ############
#################################################################
    def __read_config_file(self, default_settings, section, config_filename=None):
        """legge il file di configurazione e ne ricava le impostazioni"""
        # inizializza il parser
        config = configparser.RawConfigParser()
        # controllo se la verifica avviene su di un file temporaneo
        if config_filename is not None: file_cfg = config_filename
        else: file_cfg = get_config_file()
        # verifico l'esistenza del file da leggere
        if not os.path.exists(file_cfg):
            logger.error("File di configurazione '%s' non esistente" % file_cfg)
            return None
        # parsing del file di configurazione
        logger.debug("Parsing del file di configurazione '{0}', sezione '{1}'".format(file_cfg, section))
        config.read(file_cfg)
        # lettura dei valori e mappatura dei valori
        for key, value in default_settings.items():
           if type(value) is bool: 
               default_settings[key] = config.getboolean(section, key)
           elif type(value) is float:
               default_settings[key] = config.getfloat(section, key)
           elif type(value) is int:
               default_settings[key] = config.getint(section, key)
           elif type(value) is str:
               default_settings[key] = config.get(section, key)
           elif type(value) is list:
               default_settings[key] = eval(config.get(section, key))
           else:
               logger.error(_("Valore '{0}' del parametro '{1}' di configurazione non riconosciuto").format(value, key))
        return default_settings

    def __write_config_file(self, default_settings, section, config_filename=None):
        """memorizza sul file di configurazione le impostazioni"""
        # inizializza il parser
        config = configparser.RawConfigParser()
        # se nessun file è passato come parametro usa il file di configurazione principale
        if config_filename is None: config_filename = get_config_file()
        # verifico l'esistenza delle cartelle in cui creare il file
        if not os.path.exists(get_config_path()): 
            os.mkdir(get_config_path())
            if not os.path.exists(get_archive_path()):
                os.mkdir(get_archive_path())
        # parsing del file
        logger.debug(_("Parsing del file di configurazione, sezione '{0}'").format(config_filename))
        try:
            config.read(config_filename)
        except configparser.MissingSectionHeaderError:
            logger.error("Impossibile efffettuare il Parsing del file, sezione '{0}' mancante".format(section))
        # aggiungi la sezione se non esiste già
        if not config.has_section(section):
            config.add_section(section)
        # scrivi i valori della sezione corrente
        for key, value in default_settings.items():
           if value is True: value = "true"
           elif value is False: value = "false"
           elif type(value) is int: value = str(value)
           elif type(value) is float: value = float(value)
           elif type(value) is str: value = value
           elif type(value) is list: value = repr(value)
           else: value = str(value)
           # imposta il valore per la sezione e il parametro corrente
           config.set(section, key, value)
        # scriviamo il file di configurazione
        with open(config_filename, 'wb') as configfile:
            config.write(configfile)
            logger.debug(_("Scrittura del file di configurazione '{0}', sezione '{1}'").format(configfile, section))
        return True

    def get_main_settings(self, default_settings, section):
        """Legge i file di configurazione e costruisce il dizionario 
           delle impostazioni per l'interfaccia grafica e libreria preventa_lib"""
        # verifica se esiste già un file di configurazione
        if os.path.isfile(get_config_file()):
            try:
                default_settings = self.__read_config_file(default_settings, section, get_config_file())
            except:
                if not self.message(self, "errore", "File di configurazione '%s' danneggiato. Impossibile caricare le impostazioni." % get_config_file()):
                    self.__write_config_file(default_settings, section, get_config_file())
        #se il file di configurazione non esiste lo crea 
        else:
            self.__write_config_file(UI_DEFAULT_SETTINGS, SECTION_UI)
            self.__write_config_file(PREVENTA_LIB_DEFAULT_SETTINGS, SECTION_LIB)
        return default_settings

    def get_file_archivio(self):
        """Verifica se il file di archivio ha il percorso corretto"""
        file_archivio = self.computo_settings["main_archive_filename"]
        if os.path.isfile(file_archivio):
            return file_archivio
        else:
            return None

    def get_file_listino(self):
        """Verifica se il file di listini METEL ha il percorso corretto"""
        file_metel = self.computo_settings["metel_archive_filename"]
        return file_metel

#################################################################
########### PULSANTI DI LANCIO APPLICAZIONI E FUNZIONI ##########
#################################################################
    def computo_in_memory(self, widget, data=None):
        """Open in the RAM memory the Computo window for preventares."""
        # Questa opzione avvia una finestra con computo memorizzato nella RAM ma non su di un file fisico!!!
        # avvia lo splashscreen ed eliminalo dopo 3000 miliseconds
        splashscreen= SplashScreenWindow()
        GObject.timeout_add(3000, splashscreen.destroy)
        computo = ComputoWindow.ComputoWindow(filename=None, 
                                filearchivio = self.get_file_archivio(),
                                filelistino = self.get_file_listino(),
                                preventa_settings = self.preventa_settings, 
                                computo_settings = self.computo_settings)
        self.hide()
        GObject.idle_add(computo.show)
        Gtk.main()
        self.show()

    def new_computo(self, widget, data=None):
        """Open the a New Computo window for preventares."""
        dialog = FileChooserDialog("Salva un nuovo preventivo", "new", 
                     default_path=self.computo_settings["default_working_directory"])
        FileName = dialog.run()
        if FileName is not None:
            root, ext = os.path.splitext(FileName)
            if ext != ".db":
                FileName =  root + ".db"
            # avvia lo splashscreen ed eliminalo dopo 3000 miliseconds
            splashscreen= SplashScreenWindow()
            GObject.timeout_add(3000, splashscreen.destroy)
            computo = ComputoWindow.ComputoWindow(filename=FileName, 
                                    filearchivio = self.get_file_archivio(),
                                    filelistino = self.get_file_listino(),
                                    preventa_settings = self.preventa_settings, 
                                    computo_settings = self.computo_settings)
            self.hide()
            GObject.idle_add(computo.show)
            Gtk.main()
            self.show()

    def open_computo(self, widget, data=None):
        """Open the Computo window for preventares."""
        dialog = FileChooserDialog("Apri un preventivo esistente", "open", 
                     default_path=self.computo_settings["default_working_directory"])
        FileName = dialog.run()
        if FileName is not None:
            # avvia lo splashscreen ed eliminalo dopo 3000 miliseconds
            splashscreen= SplashScreenWindow()
            GObject.timeout_add(3000, splashscreen.destroy)
            computo = ComputoWindow.ComputoWindow(filename=FileName, 
                                    filearchivio = self.get_file_archivio(), 
                                    filelistino = self.get_file_listino(),
                                    preventa_settings= self.preventa_settings, 
                                    computo_settings=self.computo_settings)
            self.hide()
            GObject.idle_add(computo.show)
            Gtk.main()
            self.show()

    def open_archivio(self, widget, data=None):
        """Open the Archivio window for preventares."""
        # avvia lo splashscreen ed eliminalo dopo 3000 miliseconds
        splashscreen= SplashScreenWindow()
        GObject.timeout_add(3000, splashscreen.destroy)
        archivio = ComputoWindow.ComputoWindow(filename=self.get_file_archivio(), 
                                               filearchivio = None,
                                               filelistino = self.get_file_listino(),
                                               preventa_settings = self.preventa_settings, 
                                               computo_settings = self.computo_settings)
        self.hide()
        GObject.idle_add(archivio.show)
        Gtk.main()
        self.show()

    def open_listino(self, widget, data=None):
        """Open the Listini window for preventares."""
        # avvia lo splashscreen ed eliminalo dopo 3000 miliseconds
        splashscreen= SplashScreenWindow()
        GObject.timeout_add(3000, splashscreen.destroy)
        # apri il listino
        listino = MetelWindow.MetelWindow(filename= self.get_file_listino())
        self.hide()
        GObject.idle_add(listino.show)
        Gtk.main()
        self.show()

    def computo_settings(self, widget, data=None):
        """"""
        if self.message(self, "avviso", "Attenzione si stanno per modificare importanti impostazioni del programma!"):
            dialog = OptionChooserDialog.Dialog("Impostazioni del Programma", 
                                                self.computo_settings,
                                                op_keys_names=UI_SETTINGS_TRANSLATION)
            self.computo_settings = dialog.run()
            self.__write_config_file(self.computo_settings, SECTION_UI)
        else: 
            return None

    def preventa_settings(self, widget, data=None):
        """"""
        if self.message(self, "avviso", "Attenzione si stanno per modificare importanti impostazioni del database!"):
            dialog = OptionChooserDialog.Dialog("Impostazioni del Database", 
                                                self.preventa_settings, 
                                                op_keys_names=None)
            self.preventa_settings = dialog.run()
            self.__write_config_file(self.preventa_settings, SECTION_LIB)
        else: 
            return None

    def quit(self, widget, data=None):
        """Signal handler for closing the PreventaresWindow."""
        self.destroy()

    def on_destroy(self, widget, data=None):
        """Called when the PreventaresWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def about(self, widget, data=None):
        """Display the about box for preventares."""
        about = AboutPreventaresDialog.AboutPreventaresDialog()
        response = about.run()
        about.destroy()

    def preferences(self, widget, data=None):
        """Display the printing settings window for preventares."""
        # imposta il file per il salvataggio delle impostazioni
        settings_file_name = get_print_settings_file() 
        # carica le impostazioni di stampa da file (se salvate)
        try:
            pr_set = Gtk.print_settings_new_from_file(settings_file_name)
        except:
            pr_set = None
        # apri l'interfaccia delle opzioni di stampa
        print_opt = PrintoptionDialog.PrintoptionDialog(print_settings=pr_set)
        response = print_opt.run()
        if response == Gtk.ResponseType.OK:
            # Recupera le impostazioni di stampa da usare x la procedura di stampa
            print_settings = print_opt.print_settings
            # salva le impostazioni di stampa (salva le impostazioni scelte dall'utente)
            print_settings.to_file(settings_file_name)
        print_opt.destroy()

    def help(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri('preferences'))

################################################################################
###  FINESTRA MESSAGGI UTENTE ##################################################
################################################################################
    def message(self, widget, tipo, testo, data=None):
        """display a message windows"""
        buttons = Gtk.ButtonsType.OK_CANCEL
        if tipo == "info":
           tipo = Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        elif tipo == "avviso":
           tipo = Gtk.MessageType.WARNING
        elif tipo == "domanda":
           tipo = Gtk.MessageType.QUESTION
        elif tipo == "errore":
           tipo = Gtk.MessageType.ERROR
           buttons = Gtk.ButtonsType.CLOSE
        else:
           tipo == Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        message = Gtk.MessageDialog(parent=None, flags=0, type=tipo, buttons = buttons, message_format=testo)
        message.format_secondary_text(_("Premere un pulsante per continuare"))
        response = message.run()
        if response == Gtk.ResponseType.OK:
           response = True
        else:
           response = False 
        message.destroy()
        return response

################################################################################
# Splashscreen visualizzata prima dell'avvio del programma #####################
################################################################################
class SplashScreenWindow(Gtk.Window):
    def __init__(self, image_filename=None, data=None):
        self.win = Gtk.Window()
        self.win.set_position(Gtk.WindowPosition.CENTER)
        self.win.set_modal(True)
        self.win.set_decorated(False)
        # path dell'immagine di sfondo
        if image_filename is not None and os.path.isfile(image_filename):
            path = image_filename
        else: 
            path = get_data_file('media', 'splashscreen.png')
        #crea l'immagine 
        image = Gtk.Image()
        image.set_from_file(path)
        self.win.add(image)
        # show everything
        self.win.show_all()
        while Gtk.events_pending():
            Gtk.main_iteration()

    def destroy(self):
        self.win.destroy()
